@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,18 @@ raise it for the given source code reference.
 """
 
 def formatOutput( e ):
-    reason, ( filename, lineno, colno, message ) = e.args
+    if len( e.args ) > 1:
+        reason, ( filename, lineno, colno, message ) = e.args
+
+        if message is None and colno is not None:
+            colno = None
+    else:
+        reason, = e.args
+
+        filename = None
+        lineno = None
+        colno = None
+        message = None
 
     if colno is not None:
         colno = colno - len( message ) + len( message.lstrip() )
@@ -67,28 +78,38 @@ def formatOutput( e ):
          )
 
 
-def raiseSyntaxError( reason, source_ref, col_offset = None, display_file = True, display_line = True ):
-    if display_file and display_line:
+def raiseSyntaxError( reason, source_ref, col_offset = None, display_file = True,
+                      display_line = True, source_line = None ):
+    def readSource():
         source = open( source_ref.getFilename(), 'rU' ).readlines()
 
+        return source[ source_ref.getLineNumber() - 1 ]
+
+    if display_file and display_line:
+        if source_line is None:
+            source_line = readSource()
+
         raise SyntaxError(
             reason,
             (
                 source_ref.getFilename(),
                 source_ref.getLineNumber(),
                 col_offset,
-                source[ source_ref.getLineNumber() - 1 ],
+                source_line
             )
         )
     else:
         if source_ref is not None:
+            if source_line is None and display_line:
+                source_line = readSource()
+
             raise SyntaxError(
                 reason,
                 (
                     source_ref.getFilename(),
                     source_ref.getLineNumber(),
                     None,
-                    source[ source_ref.getLineNumber() - 1 ] if display_line else None,
+                    source_line
                 )
             )
         else:
