@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,13 +26,14 @@ template_constants_reading = """
 // a PyCObject pointing to NULL. It's address is unique, and that's enough.
 PyObject *_sentinel_value = NULL;
 
-PyModuleObject *module_builtin = NULL;
-
 %(constant_declarations)s
 
 static void __initConstants( void )
 {
-    UNSTREAM_INIT();
+    if ( %(needs_pickle)s )
+    {
+        UNSTREAM_INIT();
+    }
 
 %(constant_inits)s
 }
@@ -49,13 +50,6 @@ void _initConstants( void )
 #endif
         assert( _sentinel_value );
 
-#if PYTHON_VERSION < 300
-        module_builtin = (PyModuleObject *)PyImport_ImportModule( "__builtin__" );
-#else
-        module_builtin = (PyModuleObject *)PyImport_ImportModule( "builtins" );
-#endif
-        assert( module_builtin );
-
         __initConstants();
     }
 }
