@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,7 +28,7 @@ NUITKA_MAY_BE_UNUSED static PyObject *_MAKE_DICT%(pair_count)d( EVAL_ORDERED_%(a
 
     if (unlikely( result == NULL ))
     {
-        throw _PythonException();
+        throw PythonException();
     }
 
 %(add_elements_code)s
@@ -48,6 +48,6 @@ template_add_dict_element_code = """\
 
         if (unlikely( status == -1 ))
         {
-            throw _PythonException();
+            throw PythonException();
         }
     }"""
