@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -92,8 +92,15 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
         %(fparse_function_identifier)s,
         %(mparse_function_identifier)s,
         %(function_name_obj)s,
+#if PYTHON_VERSION >= 330
+        %(function_qualname_obj)s,
+#endif
         %(code_identifier)s,
         %(defaults)s,
+#if PYTHON_VERSION >= 300
+        %(kwdefaults)s,
+        %(annotations)s,
+#endif
         %(module_identifier)s,
         %(function_doc)s,
         _python_context,
@@ -109,39 +116,22 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
         %(fparse_function_identifier)s,
         %(mparse_function_identifier)s,
         %(function_name_obj)s,
+#if PYTHON_VERSION >= 330
+        %(function_qualname_obj)s,
+#endif
         %(code_identifier)s,
         %(defaults)s,
+#if PYTHON_VERSION >= 300
+        %(kwdefaults)s,
+        %(annotations)s,
+#endif
         %(module_identifier)s,
         %(function_doc)s
     );
 }
 """
 
-genfunc_yielder_with_return_template = """
-static void %(function_identifier)s_context( Nuitka_GeneratorObject *generator )
-{
-    try {
-        // Make context accessible if one is used.
-%(context_access)s
-
-        // Local variable inits
-%(function_var_inits)s
-
-        // Actual function code.
-%(function_body)s
-    }
-    catch ( GeneratorReturnException & )
-    {
-        PyErr_SetNone( PyExc_StopIteration );
-    }
-
-    // TODO: Won't return, we should tell the compiler about that.
-    generator->m_yielded = NULL;
-    swapFiber( &generator->m_yielder_context, &generator->m_caller_context );
-}
-"""
-
-genfunc_yielder_without_return_template = """
+genfunc_yielder_template = """
 static void %(function_identifier)s_context( Nuitka_GeneratorObject *generator )
 {
     {
@@ -161,9 +151,6 @@ static void %(function_identifier)s_context( Nuitka_GeneratorObject *generator )
 }
 """
 
-genfunc_yielder_return_template = """\
-throw GeneratorReturnException();"""
-
 frame_guard_genfunc_template = """\
 static PyFrameObject *frame_%(frame_identifier)s = NULL;
 
@@ -210,7 +197,7 @@ try
 
     PyErr_SetNone( PyExc_StopIteration );
 }
-catch ( _PythonException &_exception )
+catch ( PythonException &_exception )
 {
     if ( !_exception.hasTraceback() )
     {
@@ -276,7 +263,7 @@ static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
 
         return result;
     }
-    catch ( _PythonException &_exception )
+    catch ( PythonException &_exception )
     {
         _exception.toPython();
 
