@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -51,7 +51,9 @@ int main( int argc, char *argv[] )
     setCommandLineParameters( argc, argv );
 
     // Initialize the constant values used.
+    _initBuiltinModule();
     _initConstants();
+    _initBuiltinOriginalValues();
 
     // Initialize the compiled types of Nuitka.
     PyType_Ready( &Nuitka_Generator_Type );
@@ -70,17 +72,15 @@ int main( int argc, char *argv[] )
 
     patchInspectModule();
 
+    patchBuiltinModule();
+
     // Execute the "__main__" module init function.
-    MOD_INIT_NAME( __main__)();
+    MOD_INIT_NAME( __main__ )();
 
     if ( ERROR_OCCURED() )
     {
-        assertFrameObject( frame___main__ );
-        assert( frame___main__->f_back == NULL );
-
-        // Cleanup code may need a frame, so put it back.
-        Py_INCREF( frame___main__ );
-        PyThreadState_GET()->frame = frame___main__;
+        // Cleanup code may need a frame, so put one back.
+        PyThreadState_GET()->frame = MAKE_FRAME( %(code_identifier)s, _module___main__ );
 
         PyErr_PrintEx( 0 );
         Py_Exit( 1 );
@@ -99,6 +99,7 @@ module_header_template = """\
 MOD_INIT_DECL( %(module_identifier)s );
 
 extern PyObject *_module_%(module_identifier)s;
+extern PyDictObject *_moduledict_%(module_identifier)s;
 
 class PyObjectGlobalVariable_%(module_identifier)s
 {
@@ -112,26 +113,26 @@ class PyObjectGlobalVariable_%(module_identifier)s
 
         PyObject *asObject0() const
         {
-            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
+            PyObject *result = GET_STRING_DICT_VALUE( _moduledict_%(module_identifier)s, *this->var_name );
 
-            if (likely( entry->me_value != NULL ))
+            if (likely( result != NULL ))
             {
-                assertObject( entry->me_value );
+                assertObject( result );
 
-                return entry->me_value;
+                return result;
             }
 
-            entry = GET_PYDICT_ENTRY( module_builtin, *this->var_name );
+            result = GET_STRING_DICT_VALUE( dict_builtin, *this->var_name );
 
-            if (likely( entry->me_value != NULL ))
+            if (likely( result != NULL ))
             {
-                assertObject( entry->me_value );
+                assertObject( result );
 
-                return entry->me_value;
+                return result;
             }
 
             PyErr_Format( PyExc_NameError, "global name '%%s' is not defined", Nuitka_String_AsString( (PyObject *)*this->var_name ) );
-            throw _PythonException();
+            throw PythonException();
         }
 
         PyObject *asObject() const
@@ -153,71 +154,31 @@ class PyObjectGlobalVariable_%(module_identifier)s
             }
         }
 
-        void assign0( PyObject *value ) const
+        void del( bool tolerant ) const
         {
-            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
+            int status = PyDict_DelItem( (PyObject *)_moduledict_%(module_identifier)s, (PyObject *)*this->var_name );
 
-            // Values are more likely set than not set, in that case speculatively try the
-            // quickest access method.
-            if (likely( entry->me_value != NULL ))
-            {
-                PyObject *old = entry->me_value;
-                entry->me_value = INCREASE_REFCOUNT( value );
-
-                Py_DECREF( old );
-            }
-            else
-            {
-                DICT_SET_ITEM( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name, value );
-            }
-        }
-
-        void assign1( PyObject *value ) const
-        {
-            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
-
-            // Values are more likely set than not set, in that case speculatively try the
-            // quickest access method.
-            if (likely( entry->me_value != NULL ))
-            {
-                PyObject *old = entry->me_value;
-                entry->me_value = value;
-
-                Py_DECREF( old );
-            }
-            else
-            {
-                DICT_SET_ITEM( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name, value );
-
-                Py_DECREF( value );
-            }
-        }
-
-        void del() const
-        {
-            int status = PyDict_DelItem( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name );
-
-            if (unlikely( status == -1 ))
+            if (unlikely( status == -1 && tolerant == false ))
             {
                 PyErr_Format( PyExc_NameError, "global name '%%s' is not defined", Nuitka_String_AsString( (PyObject *)*this->var_name ) );
-                throw _PythonException();
+                throw PythonException();
             }
         }
 
         bool isInitialized( bool allow_builtins = true ) const
         {
-            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
+            PyObject *result = GET_STRING_DICT_VALUE( _moduledict_%(module_identifier)s, *this->var_name );
 
-            if (likely( entry->me_value != NULL ))
+            if (likely( result ))
             {
                 return true;
             }
 
             if ( allow_builtins )
             {
-                entry = GET_PYDICT_ENTRY( module_builtin, *this->var_name );
+                result = GET_STRING_DICT_VALUE( dict_builtin, *this->var_name );
 
-                return entry->me_value != NULL;
+                return result != NULL;
             }
             else
             {
@@ -226,9 +187,12 @@ class PyObjectGlobalVariable_%(module_identifier)s
         }
 
     private:
+
         Nuitka_StringObject **var_name;
 };
 
+// Declarations from this module to other modules if any.
+%(extra_declarations)s
 """
 
 module_body_template = """
@@ -245,6 +209,7 @@ module_body_template = """
 // time.
 
 PyObject *_module_%(module_identifier)s;
+PyDictObject *_moduledict_%(module_identifier)s;
 
 // The module level variables.
 %(module_globals)s
@@ -255,9 +220,6 @@ PyObject *_module_%(module_identifier)s;
 // The module function definitions.
 %(module_functions_code)s
 
-// Frame object of the module.
-static PyFrameObject *frame_%(module_identifier)s;
-
 #if PYTHON_VERSION >= 300
 static struct PyModuleDef _moduledef =
 {
@@ -277,14 +239,20 @@ static struct PyModuleDef _moduledef =
 
 #if _MODULE_UNFREEZER
 // For embedded modules, to be unpacked. Used by main program/package only
-extern void registerMetaPathBasedUnfreezer( struct _inittab *_frozes_modules );
+extern void registerMetaPathBasedUnfreezer( struct _inittab *_frozen_modules );
 
 // Our own inittab for lookup of "frozen" modules, i.e. the ones included in this binary.
-static struct _inittab _frozes_modules[] =
+static struct _inittab _frozen_modules[] =
 {
 %(module_inittab)s
     { NULL, NULL }
 };
+
+// For loader attribute.
+#if PYTHON_VERSION >= 330
+extern PyObject *loader_frozen_modules;
+#endif
+
 #endif
 
 #ifdef _NUITKA_EXE
@@ -313,6 +281,7 @@ MOD_INIT_DECL( %(module_identifier)s )
     // because that's how we are going to get called here.
 
     // Initialize the constant values used.
+    _initBuiltinModule();
     _initConstants();
 
     // Initialize the compiled types of Nuitka.
@@ -322,10 +291,12 @@ MOD_INIT_DECL( %(module_identifier)s )
     PyType_Ready( &Nuitka_Frame_Type );
 
     patchInspectModule();
+
+    patchBuiltinModule();
 #endif
 
 #if _MODULE_UNFREEZER
-    registerMetaPathBasedUnfreezer( _frozes_modules );
+    registerMetaPathBasedUnfreezer( _frozen_modules );
 #endif
 
     // puts( "in init%(module_identifier)s" );
@@ -346,11 +317,13 @@ MOD_INIT_DECL( %(module_identifier)s )
     _module_%(module_identifier)s = PyModule_Create( &_moduledef );
 #endif
 
+    _moduledict_%(module_identifier)s = (PyDictObject *)((PyModuleObject *)_module_%(module_identifier)s)->md_dict;
+
     assertObject( _module_%(module_identifier)s );
 
 #ifndef _NUITKA_MODULE
-// TODO: Seems to work for Python2.7 as well, and maybe even useful. To be
-// investigated in separate tests.
+// Seems to work for Python2.7 out of the box, but for Python3.2, the module doesn't automatically enter
+// "sys.modules" with the object that it should, so do it manually.
 #if PYTHON_VERSION >= 300
     {
         int r = PyObject_SetItem( PySys_GetObject( (char *)"modules" ), %(module_name_obj)s, _module_%(module_identifier)s );
@@ -370,7 +343,7 @@ MOD_INIT_DECL( %(module_identifier)s )
     {
         PyObject *value = ( PyObject *)module_builtin;
 
-#ifndef _NUITKA_MODULE
+#ifdef _NUITKA_EXE
         if ( _module_%(module_identifier)s != _module___main__ )
         {
             value = PyModule_GetDict( value );
@@ -385,85 +358,24 @@ MOD_INIT_DECL( %(module_identifier)s )
         assert( res == 0 );
     }
 
-#if PYTHON_VERSION >= 300
-    {
-#ifndef __NUITKA_NO_ASSERT__
-        int res =
+#if PYTHON_VERSION >= 330
+#if _MODULE_UNFREEZER
+    PyDict_SetItem( module_dict, _python_str_plain___loader__, loader_frozen_modules );
+#else
+    PyDict_SetItem( module_dict, _python_str_plain___loader__, Py_None );
 #endif
-            PyDict_SetItem( module_dict, _python_str_plain___cached__, Py_None );
-
-        assert( res == 0 );
-    }
 #endif
 
-    frame_%(module_identifier)s = MAKE_FRAME( %(code_identifier)s, _module_%(module_identifier)s );
-
-    // Set module frame as the currently active one.
-    FrameGuardLight frame_guard( &frame_%(module_identifier)s );
-
-    // Push the new frame as the currently active one.
-    pushFrameStack( frame_%(module_identifier)s );
-
-    // Initialize the standard module attributes.
+    // Temporary variables if any
 %(module_inits)s
 
     // Module code
-    try
-    {
-        // To restore the initial exception, could be made dependent on actual try/except statement
-        // as it is done for functions/classes already.
-        FrameExceptionKeeper _frame_exception_keeper;
 %(module_code)s
-    }
-    catch ( _PythonException &_exception )
-    {
-        if ( !_exception.hasTraceback() )
-        {
-            _exception.setTraceback( MAKE_TRACEBACK( frame_guard.getFrame() ) );
-        }
-        else
-        {
-            _exception.addTraceback( frame_guard.getFrame0() );
-        }
 
-        _exception.toPython();
-    }
-
-    // Pop the frame from the frame stack, we are done here.
-    assert( PyThreadState_GET()->frame == frame_%(module_identifier)s );
-    PyThreadState_GET()->frame = INCREASE_REFCOUNT_X( PyThreadState_GET()->frame->f_back );
-
-    // puts( "out init%(module_identifier)s" );
-
-    return MOD_RETURN_VALUE( _module_%(module_identifier)s );
+   return MOD_RETURN_VALUE( _module_%(module_identifier)s );
 }
 """
 
-module_init_no_package_template = """\
-    _mvar_%(module_identifier)s___doc__.assign0( %(doc_identifier)s );
-    _mvar_%(module_identifier)s___file__.assign0( %(filename_identifier)s );
-#if %(is_package)d
-    _mvar_%(module_identifier)s___path__.assign0( %(path_identifier)s );
-#endif
-"""
-
-module_init_in_package_template = """\
-    _mvar_%(module_identifier)s___doc__.assign0( %(doc_identifier)s );
-    _mvar_%(module_identifier)s___file__.assign0( %(filename_identifier)s );
-#if %(is_package)d
-    _mvar_%(module_identifier)s___path__.assign0( %(path_identifier)s );
-#endif
-
-    // The package must already be imported.
-    assertObject( _module_%(package_identifier)s );
-
-    SET_ATTRIBUTE(
-        _module_%(module_identifier)s,
-        _module_%(package_identifier)s,
-        %(module_name)s
-    );
-"""
-
 template_header_guard = """\
 #ifndef %(header_guard_name)s
 #define %(header_guard_name)s
