@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,51 +22,41 @@ and cross-module optimizations are the most difficult to tackle.
 """
 
 from .NodeBases import (
-    CPythonClosureGiverNodeBase,
-    CPythonChildrenHaving,
-    CPythonClosureTaker
+    ClosureGiverNodeBase,
+    ChildrenHavingMixin
 )
 
 from .IndicatorMixins import MarkContainsTryExceptIndicator
 
-from nuitka import (
-    Variables,
-    Utils
-)
+from nuitka import Variables, Utils
 
 from nuitka.oset import OrderedSet
 
-class CPythonModule( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureGiverNodeBase, \
-                     MarkContainsTryExceptIndicator ):
+class PythonModule( ChildrenHavingMixin, ClosureGiverNodeBase,
+                    MarkContainsTryExceptIndicator ):
     """ Module
 
         The module is the only possible root of a tree. When there are many modules
         they form a forrest.
     """
 
-    kind = "MODULE"
+    kind = "PYTHON_MODULE"
 
     named_children = ( "body", )
 
     def __init__( self, name, package, source_ref ):
         assert type(name) is str, type(name)
-        assert "." not in name
+        assert "." not in name, name
         assert package is None or ( type( package ) is str and package != "" )
 
-        CPythonClosureGiverNodeBase.__init__(
+        ClosureGiverNodeBase.__init__(
             self,
             name        = name,
             code_prefix = "module",
             source_ref  = source_ref
         )
 
-        CPythonClosureTaker.__init__(
-            self,
-            provider      = self,
-            early_closure = True
-        )
-
-        CPythonChildrenHaving.__init__(
+        ChildrenHavingMixin.__init__(
             self,
             values = {}
         )
@@ -74,7 +64,6 @@ class CPythonModule( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureG
         MarkContainsTryExceptIndicator.__init__( self )
 
         self.package = package
-        self.doc = None
 
         self.variables = set()
 
@@ -82,20 +71,36 @@ class CPythonModule( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureG
         self.functions = OrderedSet()
 
     def getDetails( self ):
-        return { "filename" : self.source_ref.getFilename()  }
+        return {
+            "filename" : self.source_ref.getFilename(),
+            "package"  : self.package,
+            "name"     : self.name
+        }
 
-    getBody = CPythonChildrenHaving.childGetter( "body" )
-    setBody = CPythonChildrenHaving.childSetter( "body" )
+    def asXml( self ):
+        # The class is new style, false alarm: pylint: disable=E1002
+        result = super( PythonModule, self ).asXml()
+
+        for function_body in self.functions:
+            result.append( function_body.asXml() )
+
+        return result
+
+    getBody = ChildrenHavingMixin.childGetter( "body" )
+    setBody = ChildrenHavingMixin.childSetter( "body" )
+
+    def isPythonModule( self ):
+        return True
+
+    def getParent( self ):
+        assert False
+
+    def getParentVariableProvider( self ):
+        return None
 
     def getVariables( self ):
         return self.variables
 
-    def getDoc( self ):
-        return self.doc
-
-    def setDoc( self, doc ):
-        self.doc = doc
-
     def getFilename( self ):
         return self.source_ref.getFilename()
 
@@ -135,8 +140,12 @@ class CPythonModule( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureG
         return result
 
     def isEarlyClosure( self ):
+        # Modules should immediately closure variables on use, pylint: disable=R0201
         return True
 
+    def isMainModule( self ):
+        return False
+
     def getCodeName( self ):
         return "module_" + self.getFullName().replace( ".", "__" ).replace( "-", "_" )
 
@@ -148,19 +157,53 @@ class CPythonModule( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureG
     def getFunctions( self ):
         return self.functions
 
+    def getOutputFilename( self ):
+        main_filename = self.getFilename()
 
-class CPythonPackage( CPythonModule ):
-    kind = "PACKAGE"
+        if main_filename.endswith( ".py" ):
+            return main_filename[:-3]
+        else:
+            return main_filename
+
+
+class PythonMainModule( PythonModule ):
+    kind = "PYTHON_MAIN_MODULE"
+
+    def __init__( self, main_added, source_ref ):
+        PythonModule.__init__(
+            self,
+            name        = "__main__",
+            package     = None,
+            source_ref  = source_ref
+        )
+
+        self.main_added = main_added
+
+    def isMainModule( self ):
+        return True
+
+    def isMainAdded( self ):
+        return self.main_added
+
+    def getOutputFilename( self ):
+        if self.main_added:
+            return Utils.dirname( self.getFilename() )
+        else:
+            return PythonModule.getOutputFilename( self )
+
+
+class PythonPackage( PythonModule ):
+    kind = "PYTHON_PACKAGE"
 
     def __init__( self, name, package, source_ref ):
         assert name
 
-        CPythonModule.__init__(
+        PythonModule.__init__(
             self,
             name       = name,
             package    = package,
             source_ref = source_ref
         )
 
-    def getPathAttribute( self ):
-        return [ Utils.dirname( self.getFilename() ) ]
+    def getOutputFilename( self ):
+        return Utils.dirname( self.getFilename() )
