@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,14 +23,13 @@ These classes provide the generic base classes available for nodes.
 
 
 from nuitka.odict import OrderedDict
+from nuitka.oset import OrderedSet
 
 from nuitka import (
     Tracing,
     TreeXML
 )
 
-from . import UsageCheck
-
 from nuitka.__past__ import iterItems
 
 lxml = TreeXML.lxml
@@ -39,15 +38,6 @@ class NodeCheckMetaClass( type ):
     kinds = set()
 
     def __new__( mcs, name, bases, dictionary ):
-        # Merge the tags with the base classes in a non-overriding
-        # way, instead add them up.
-        if "tags" not in dictionary:
-            dictionary[ "tags" ] = ()
-
-        for base in bases:
-            if hasattr( base, "tags" ):
-                dictionary[ "tags" ] += getattr( base, "tags" )
-
         assert len( bases ) == len( set( bases ) )
 
         # Uncomment this for debug view of class tags.
@@ -79,18 +69,11 @@ class NodeCheckMetaClass( type ):
             # Automatically add checker methods for everything to the common base class
             checker_method = "is" + kind_to_name_part
 
-            if name.startswith( "CPython" ):
-                checker_method = checker_method.replace( "CPython", "" )
-
             def checkKind( self ):
                 return self.kind == kind
 
-            if not hasattr( CPythonNodeBase, checker_method ):
-                setattr( CPythonNodeBase, checker_method, checkKind )
-
-            # Tags mechanism, so node classes can be tagged with inheritance or freely,
-            # the "tags" attribute is not overloaded, but added. Absolutely not obvious
-            # and a trap set for the compiler by itself.
+            if not hasattr( NodeBase, checker_method ):
+                setattr( NodeBase, checker_method, checkKind )
 
         type.__init__( mcs, name, bases, dictionary )
 
@@ -98,13 +81,11 @@ class NodeCheckMetaClass( type ):
 
 # For Python2/3 compatible source, we create a base class that has the metaclass used and
 # doesn't require making a choice.
-CPythonNodeMetaClassBase = NodeCheckMetaClass( "CPythonNodeMetaClassBase", (object, ), {} )
+NodeMetaClassBase = NodeCheckMetaClass( "NodeMetaClassBase", (object, ), {} )
 
-class CPythonNodeBase( CPythonNodeMetaClassBase ):
+class NodeBase( NodeMetaClassBase ):
     kind = None
 
-    tags = ()
-
     # Must be overloaded by expressions.
     value_friend_maker = None
 
@@ -139,41 +120,29 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         return "%s at %s" % ( self.kind, self.source_ref.getAsString() )
 
     def getDetails( self ):
-        """ Details of the node, intented for use in __repr__ and dumps.
+        """ Details of the node, intended for use in __repr__ and dumps.
 
         """
         # Virtual method, pylint: disable=R0201
         return {}
 
     def getDetail( self ):
-        """ Details of the node, intented for use in __repr__ and graphical display.
+        """ Details of the node, intended for use in __repr__ and graphical display.
 
         """
         # Virtual method, pylint: disable=R0201
-        return self.getDetails()
+        return str( self.getDetails() )[1:-1]
 
     def getParent( self ):
         """ Parent of the node. Every node except modules have to have a parent.
 
         """
 
-        if self.parent is None and not self.isModule():
+        if self.parent is None and not self.isPythonModule():
             assert False, ( self,  self.source_ref )
 
         return self.parent
 
-    def getParentExecInline( self ):
-        """ Return the parent that is an inlined exec.
-
-        """
-
-        parent = self.getParent()
-
-        while parent is not None and not parent.isStatementExecInline():
-            parent = parent.getParent()
-
-        return parent
-
     def getParentFunction( self ):
         """ Return the parent that is a function.
 
@@ -192,7 +161,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         """
         parent = self
 
-        while not parent.isModule():
+        while not parent.isPythonModule():
             if hasattr( parent, "provider" ):
                 # After we checked, we can use it, will be much faster, pylint: disable=E1101
                 parent = parent.provider
@@ -204,7 +173,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
     def isParentVariableProvider( self ):
         # Check if it's a closure giver, in which cases it can provide variables,
         # pylint: disable=E1101
-        return isinstance( self, CPythonClosureGiverNodeBase )
+        return isinstance( self, ClosureGiverNodeBase )
 
     def getParentVariableProvider( self ):
         parent = self.getParent()
@@ -220,7 +189,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
     def asXml( self ):
         result = lxml.etree.Element(
             "node",
-            kind = self.__class__.__name__.replace( "CPython", "" ),
+            kind = self.__class__.__name__,
             line = "%s" % self.getSourceReference().getLineNumber()
         )
 
@@ -260,9 +229,6 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         Tracing.printSeparator( level )
 
-    def isModule( self ):
-        return self.kind in ( "MODULE", "PACKAGE" )
-
     def isExpression( self ):
         return self.kind.startswith( "EXPRESSION_" )
 
@@ -290,6 +256,10 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
+    def isExpressionCall( self ):
+        # Virtual method, pylint: disable=R0201,W0613
+        return False
+
     def visit( self, context, visitor ):
         visitor( self )
 
@@ -304,19 +274,6 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201
         return ()
 
-    def getChildNodesNotTagged( self, tag ):
-        """ Get child nodes that do not have a given tag.
-
-        """
-
-        return [
-            node
-            for node in
-            self.getVisitableNodes()
-            if not node.hasTag( tag )
-        ]
-
-
     def replaceWith( self, new_node ):
         self.parent.replaceChild(
             old_node = self,
@@ -333,6 +290,12 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         return True
 
+    def mayHaveSideEffectsBool( self, constraint_collection ):
+        """ Unless we are told otherwise, everything may have a side effect. """
+        # Virtual method, pylint: disable=R0201,W0613
+
+        return True
+
     def extractSideEffects( self ):
         """ Unless defined otherwise, the expression is the side effect. """
         # Virtual method, pylint: disable=R0201,W0613
@@ -345,6 +308,13 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         return True
 
+    def willRaiseException( self, exception_type ):
+        """ Unless we are told otherwise, nothing may raise anything. """
+        # Virtual method, pylint: disable=R0201,W0613
+
+        return False
+
+
     def needsLineNumber( self ):
         return self.mayRaiseException( BaseException )
 
@@ -354,8 +324,8 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         return False
 
-    def isStatementAbortative( self ):
-        """ Is the node abortative, control flow doesn't continue after this node.  """
+    def isStatementAborting( self ):
+        """ Is the node aborting, control flow doesn't continue after this node.  """
         # Virtual method, pylint: disable=R0201
         assert self.isStatement(), self.kind
 
@@ -367,21 +337,19 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def hasTag( self, tag ):
-        return tag in self.__class__.tags
-
     def getIntegerValue( self, constraint_collection ):
         """ Node as integer value, if possible."""
         # Virtual method, pylint: disable=R0201,W0613
+        return None
 
 
-class CPythonCodeNodeBase( CPythonNodeBase ):
+class CodeNodeBase( NodeBase ):
     def __init__( self, name, code_prefix, source_ref ):
         assert name is not None
         assert " " not in name, name
         assert "<" not in name, name
 
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__( self, source_ref = source_ref )
 
         self.name = name
         self.code_prefix = code_prefix
@@ -417,19 +385,14 @@ class CPythonCodeNodeBase( CPythonNodeBase ):
 
     def getCodeName( self ):
         if self.code_name is None:
-            search = self.parent
+            provider = self.getParentVariableProvider()
+            parent_name = provider.getCodeName()
 
-            while search is not None:
-                if isinstance( search, CPythonCodeNodeBase ):
-                    break
+            uid = "_%d" % provider.getChildUID( self )
 
-                search = search.parent
+            assert isinstance( self, CodeNodeBase )
 
-            parent_name = search.getCodeName()
-
-            uid = "_%d" % search.getChildUID( self )
-
-            if isinstance( self, CPythonCodeNodeBase ) and self.name:
+            if self.name:
                 name = uid + "_" + self.name
             else:
                 name = uid
@@ -446,7 +409,7 @@ class CPythonCodeNodeBase( CPythonNodeBase ):
 
         return self.uids[ node.kind ]
 
-class CPythonChildrenHaving:
+class ChildrenHavingMixin:
     named_children = ()
 
     def __init__( self, values ):
@@ -492,6 +455,9 @@ class CPythonChildrenHaving:
 
         return self.child_values[ name ]
 
+    def hasChild( self, name ):
+        return name in self.child_values
+
     @staticmethod
     def childGetter( name ):
         def getter( self ):
@@ -525,7 +491,7 @@ class CPythonChildrenHaving:
                 pass
             elif type( value ) is tuple:
                 result += list( value )
-            elif isinstance( value, CPythonNodeBase ):
+            elif isinstance( value, NodeBase ):
                 result.append( value )
             else:
                 raise AssertionError( self, "has illegal child", name, value, value.__class__ )
@@ -543,7 +509,7 @@ class CPythonChildrenHaving:
         return result
 
     def replaceChild( self, old_node, new_node ):
-        if new_node is not None and not isinstance( new_node, CPythonNodeBase ):
+        if new_node is not None and not isinstance( new_node, NodeBase ):
             raise AssertionError( "Cannot replace with", new_node, "old", old_node, "in", self )
 
         for key, value in self.child_values.items():
@@ -551,17 +517,30 @@ class CPythonChildrenHaving:
                 pass
             elif type( value ) is tuple:
                 if old_node in value:
-                    self.setChild(
-                        key,
-                        tuple(
-                            (val if val is not old_node else new_node)
-                            for val in
-                            value
+                    if new_node is not None:
+                        self.setChild(
+                            key,
+                            tuple(
+                                (val if val is not old_node else new_node)
+                                for val in
+                                value
+                            )
                         )
-                    )
+                    else:
+                        self.setChild(
+                            key,
+                            tuple(
+                                val
+                                for val in
+                                value
+                                if val is not old_node
+                            )
+                        )
+
+
 
                     break
-            elif isinstance( value, CPythonNodeBase ):
+            elif isinstance( value, NodeBase ):
                 if old_node is value:
                     self.setChild( key, new_node )
 
@@ -613,10 +592,10 @@ class CPythonChildrenHaving:
             raise
 
 
-class CPythonClosureGiverNodeBase( CPythonCodeNodeBase ):
+class ClosureGiverNodeBase( CodeNodeBase ):
     """ Mixin for nodes that provide variables for closure takers. """
     def __init__( self, name, code_prefix, source_ref ):
-        CPythonCodeNodeBase.__init__(
+        CodeNodeBase.__init__(
             self,
             name        = name,
             code_prefix = code_prefix,
@@ -625,6 +604,8 @@ class CPythonClosureGiverNodeBase( CPythonCodeNodeBase ):
 
         self.providing = OrderedDict()
 
+        self.keeper_variables = OrderedSet()
+
     def hasProvidedVariable( self, variable_name ):
         return variable_name in self.providing
 
@@ -654,29 +635,24 @@ class CPythonClosureGiverNodeBase( CPythonCodeNodeBase ):
     def getProvidedVariables( self ):
         return self.providing.values()
 
-    def reconsiderVariable( self, variable ):
-        # TODO: Why doesn't this fit in as well.
-        if self.isModule():
-            return
+    def getTempKeeperVariable( self ):
+        name = "keeper_%d" % len( self.keeper_variables )
 
-        assert variable.getOwner() is self
+        from nuitka import Variables
 
-        if variable.getName() in self.providing:
-            assert self.providing[ variable.getName() ] is variable, (
-                self.providing[ variable.getName() ], "is not", variable, self
-            )
+        result = Variables.TempKeeperVariable(
+            owner         = self,
+            variable_name = name
+        )
 
-            if not variable.isShared():
-                # TODO: The functions/classes should have have a clearer scope too.
-                usages = UsageCheck.getVariableUsages( self, variable )
+        self.keeper_variables.add( result )
 
-                if not usages:
-                    del self.providing[ variable.getName() ]
+        return result
 
 
-class CPythonParameterHavingNodeBase( CPythonClosureGiverNodeBase ):
+class ParameterHavingNodeBase( ClosureGiverNodeBase ):
     def __init__( self, name, code_prefix, parameters, source_ref ):
-        CPythonClosureGiverNodeBase.__init__(
+        ClosureGiverNodeBase.__init__(
             self,
             name        = name,
             code_prefix = code_prefix,
@@ -694,11 +670,9 @@ class CPythonParameterHavingNodeBase( CPythonClosureGiverNodeBase ):
         return self.parameters
 
 
-class CPythonClosureTaker:
+class ClosureTakerMixin:
     """ Mixin for nodes that accept variables from closure givers. """
 
-    tags = ( "closure_taker", )
-
     def __init__( self, provider, early_closure ):
         assert provider.isParentVariableProvider(), provider
 
@@ -709,8 +683,6 @@ class CPythonClosureTaker:
 
         self.temp_variables = set()
 
-        self.temp_keeper_count = 0
-
     def getParentVariableProvider( self ):
         return self.provider
 
@@ -773,13 +745,8 @@ class CPythonClosureTaker:
 
         return self.early_closure
 
-    def allocateTempKeeperName( self ):
-        self.temp_keeper_count += 1
 
-        return "keeper_%d" % self.temp_keeper_count
-
-
-class CPythonExpressionMixin:
+class ExpressionMixin:
     def getValueFriend( self, constraint_collection ):
         return self
 
@@ -846,11 +813,25 @@ class CPythonExpressionMixin:
         # Virtual method, pylint: disable=R0201
         return None
 
-    def getStrValue( self ):
+    def getStringValue( self, constraint_collection ):
+        """ Node as integer value, if possible."""
+        # Virtual method, pylint: disable=R0201,W0613
+        return None
+
+    def getStrValue( self, constraint_collection ):
         """ Value that "str" or "PyObject_Str" would give, if known.
 
         Otherwise it is "None" to indicate unknown.
         """
+        string_value = self.getStringValue( constraint_collection )
+
+        if string_value is not None:
+            from .NodeMakingHelpers import makeConstantReplacementNode
+
+            return makeConstantReplacementNode(
+                node     = self,
+                constant = string_value
+            )
 
         return None
 
@@ -858,9 +839,146 @@ class CPythonExpressionMixin:
         # print "onRelease", self
         pass
 
+    def computeExpressionAttribute( self, lookup_node, attribute_name, constraint_collection ):
+        # By default, an attribute lookup may change everything about the lookup source.
+        # Virtual method, pylint: disable=R0201,W0613
+        constraint_collection.removeKnowledge( lookup_node )
+
+        return lookup_node, None, None
+
+    def computeExpressionSubscript( self, lookup_node, subscript, constraint_collection ):
+        # By default, an subscript may change everything about the lookup source.
+        constraint_collection.removeKnowledge( lookup_node )
+
+        return lookup_node, None, None
+
+    def computeExpressionSlice( self, lookup_node, lower, upper, constraint_collection ):
+        # By default, a slicing may change everything about the lookup source.
+        constraint_collection.removeKnowledge( lookup_node )
+
+        return lookup_node, None, None
+
+    def computeExpressionCall( self, call_node, constraint_collection ):
+        call_node.getCalled().onContentEscapes( constraint_collection )
+
+        return call_node, None, None
+
+    def computeExpressionOperationNot( self, not_node, constraint_collection ):
+        constraint_collection.removeKnowledge( not_node )
+
+        return not_node, None, None
+
+    def onContentEscapes( self, constraint_collection ):
+        pass
+
+class CompileTimeConstantExpressionMixin( ExpressionMixin ):
+    def __init__( self ):
+        self.computed_attribute = False
+
+    def isCompileTimeConstant( self ):
+        """ Has a value that we can use at compile time.
+
+            Yes or no. If it has such a value, simulations can be applied at compile time
+            and e.g. operations or conditions, or even calls may be executed against it.
+        """
+        # Virtual method, pylint: disable=R0201
+
+        return True
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        return False
+
+    def mayHaveSideEffectsBool( self, constraint_collection ):
+        return False
+
+    def computeExpressionOperationNot( self, not_node, constraint_collection ):
+        from .NodeMakingHelpers import getComputationResult
+
+        return getComputationResult(
+            node        = not_node,
+            computation = lambda : not self.getCompileTimeConstant(),
+            description = "Compile time constant negation truth value precomputed."
+        )
 
 
-class CPythonExpressionSpecBasedComputationMixin( CPythonExpressionMixin ):
+    def computeExpressionAttribute( self, lookup_node, attribute_name, constraint_collection ):
+        if self.computed_attribute:
+            return lookup_node, None, None
+
+        value = self.getCompileTimeConstant()
+
+        from .NodeMakingHelpers import getComputationResult, isCompileTimeConstantValue
+
+        if not hasattr( value, attribute_name ) or isCompileTimeConstantValue( getattr( value, attribute_name ) ):
+
+            return getComputationResult(
+                node        = lookup_node,
+                computation = lambda : getattr( value, attribute_name ),
+                description = "Attribute lookup to %s precomputed." % attribute_name
+            )
+
+        self.computed_attribute = True
+
+        return lookup_node, None, None
+
+
+    def computeExpressionSubscript( self, lookup_node, subscript, constraint_collection ):
+        from .NodeMakingHelpers import getComputationResult
+
+        if subscript.isCompileTimeConstant():
+            return getComputationResult(
+                node        = lookup_node,
+                computation = lambda : self.getCompileTimeConstant()[ subscript.getCompileTimeConstant() ],
+                description = "Subscript of constant with constant value."
+            )
+
+        return lookup_node, None, None
+
+    def computeExpressionSlice( self, lookup_node, lower, upper, constraint_collection ):
+        from .NodeMakingHelpers import getComputationResult
+
+        # TODO: Could be happy with predictable index values and not require constants.
+        if lower is not None:
+            if upper is not None:
+                if lower.isCompileTimeConstant() and upper.isCompileTimeConstant():
+
+                    return getComputationResult(
+                        node        = lookup_node,
+                        computation = lambda : self.getCompileTimeConstant()[
+                            lower.getCompileTimeConstant() : upper.getCompileTimeConstant()
+                        ],
+                        description = "Slicing of constant with constant indexes."
+                    )
+            else:
+                if lower.isCompileTimeConstant():
+                    return getComputationResult(
+                        node        = lookup_node,
+                        computation = lambda : self.getCompileTimeConstant()[
+                            lower.getCompileTimeConstant() :
+                        ],
+                        description = "Slicing of constant with constant lower index only."
+                    )
+        else:
+            if upper is not None:
+                if upper.isCompileTimeConstant():
+                    return getComputationResult(
+                        node        = lookup_node,
+                        computation = lambda : self.getCompileTimeConstant()[
+                            : upper.getCompileTimeConstant()
+                        ],
+                        description = "Slicing of constant with constant upper index only."
+                    )
+            else:
+                return getComputationResult(
+                    node        = lookup_node,
+                    computation = lambda : self.getCompileTimeConstant()[ : ],
+                    description = "Slicing of constant with no indexes."
+                )
+
+        return lookup_node, None, None
+
+
+class ExpressionSpecBasedComputationMixin( ExpressionMixin ):
     builtin_spec = None
 
     def computeBuiltinSpec( self, given_values ):
@@ -882,28 +1000,38 @@ class CPythonExpressionSpecBasedComputationMixin( CPythonExpressionMixin ):
         )
 
 
-class CPythonExpressionChildrenHavingBase( CPythonChildrenHaving, CPythonNodeBase, CPythonExpressionMixin ):
+class ExpressionChildrenHavingBase( ChildrenHavingMixin, NodeBase, ExpressionMixin ):
     def __init__( self, values, source_ref ):
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__( self, source_ref = source_ref )
 
-        CPythonChildrenHaving.__init__(
+        ChildrenHavingMixin.__init__(
             self,
             values = values
         )
 
+class StatementChildrenHavingBase( ChildrenHavingMixin, NodeBase ):
+    def __init__( self, values, source_ref ):
+        NodeBase.__init__( self, source_ref = source_ref )
 
-class CPythonExpressionBuiltinNoArgBase( CPythonNodeBase, CPythonExpressionMixin ):
+        ChildrenHavingMixin.__init__(
+            self,
+            values = values
+        )
+
+class ExpressionBuiltinNoArgBase( NodeBase, ExpressionMixin ):
     def __init__( self, builtin_function, source_ref ):
-        CPythonNodeBase.__init__(
+        NodeBase.__init__(
             self,
             source_ref = source_ref
         )
 
         self.builtin_function = builtin_function
 
-    def computeNode( self, constraint_collection ):
+    def computeExpression( self, constraint_collection ):
         from .NodeMakingHelpers import getComputationResult
 
+        # The lamba is there for make sure that no argument parsing will reach the builtin
+        # function at all, pylint: disable=W0108
         return getComputationResult(
             node        = self,
             computation = lambda : self.builtin_function(),
@@ -911,12 +1039,12 @@ class CPythonExpressionBuiltinNoArgBase( CPythonNodeBase, CPythonExpressionMixin
         )
 
 
-class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase, \
-                                             CPythonExpressionSpecBasedComputationMixin ):
+class ExpressionBuiltinSingleArgBase( ExpressionChildrenHavingBase,
+                                      ExpressionSpecBasedComputationMixin ):
     named_children = ( "value", )
 
     def __init__( self, value, source_ref ):
-        CPythonExpressionChildrenHavingBase.__init__(
+        ExpressionChildrenHavingBase.__init__(
             self,
             values = {
                 "value" : value,
@@ -924,9 +1052,9 @@ class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase
             source_ref = source_ref
         )
 
-    getValue = CPythonExpressionChildrenHavingBase.childGetter( "value" )
+    getValue = ExpressionChildrenHavingBase.childGetter( "value" )
 
-    def computeNode( self, constraint_collection ):
+    def computeExpression( self, constraint_collection ):
         value = self.getValue()
 
         assert self.builtin_spec is not None, self
@@ -934,10 +1062,13 @@ class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase
         if value is None:
             return self.computeBuiltinSpec( () )
         else:
+            if value.willRaiseException( BaseException ):
+                return value, "new_raise", "Builtin call raises exception while building argument"
+
             return self.computeBuiltinSpec( ( value, ) )
 
 
-class CPythonSideEffectsFromChildrenMixin:
+class SideEffectsFromChildrenMixin:
     def mayHaveSideEffects( self, constraint_collection ):
         for child in self.getVisitableNodes():
             if child.mayHaveSideEffects( constraint_collection ):
