@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -109,10 +109,10 @@ class CustomOps( int ):
 
 
 print "Custom ops, to enforce chain eval order and short circuit:"
-CustomOps( 7 ) < CustomOps( 8 ) > CustomOps( 6 )
+print CustomOps( 7 ) < CustomOps( 8 ) > CustomOps( 6 )
 
 print "Custom ops, do short circuit:"
-CustomOps( 8 ) > CustomOps( 7 ) < CustomOps( 6 )
+print CustomOps( 8 ) > CustomOps( 7 ) < CustomOps( 6 )
 
 def inOperatorChain():
     print "In operator chains:"
