@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -74,7 +74,7 @@ for filename in sorted( os.listdir( "." ) ):
         continue
 
     # Skip tests that require Python 3.2 at least.
-    if filename.endswith( "32.py" ) and not python_version.startswith( b"3.2" ):
+    if filename.endswith( "32.py" ) and not python_version.startswith( b"3" ):
         continue
 
     # The overflow functions test gives syntax error on Python 3.x and can be ignored.
@@ -89,7 +89,7 @@ for filename in sorted( os.listdir( "." ) ):
     extra_flags = [ "expect_success", "remove_output" ]
 
     if active:
-        if filename == "Referencing.py":
+        if filename in ( "Referencing.py", "Referencing32.py" ):
             debug_python = os.environ[ "PYTHON" ]
             if not debug_python.endswith( "-dbg" ):
                 debug_python += "-dbg"
@@ -105,12 +105,6 @@ for filename in sorted( os.listdir( "." ) ):
             extra_flags.append( "ignore_stderr" )
             extra_flags.append( "python_debug" )
 
-        # Temporary measure, until Python3 is better supported, disable some tests, so
-        # this can be used to monitor the success of existing ones and have no regression for it.
-        if python_version.startswith( b"3.2" ) and filename[:-3] in ( "ExecEval",  ):
-            print( "Skipping malfunctional test", filename )
-            continue
-
         # Apply 2to3 conversion if necessary.
         assert type( python_version ) is bytes
 
